﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Text.RegularExpressions;
using Enyim.Caching.Configuration;
using Enyim.Caching.Memcached;
using System.Net;
using System.Reflection;
using System.Collections;

namespace MyMemcached.DAO
{
    public class Stats
    {
        public static Dictionary<string, mcacheClientExt> MClientList = new Dictionary<string, mcacheClientExt>();
        static readonly object padlock = new object();
        public static mcacheClientExt MClient(IPEndPoint EP)
        {
            lock (padlock)
            {
                if (!MClientList.ContainsKey(EP.ToString()))
                {
                    MemcachedClientConfiguration _config = new MemcachedClientConfiguration();
                    _config.Servers.Add(EP);
                    _config.NodeLocator = typeof(DefaultNodeLocator);
                    _config.KeyTransformer = new DefaultKeyTransformer();
                    _config.Transcoder = new DefaultTranscoder();
                    MClientList.Add(EP.ToString(), new mcacheClientExt(_config));
                }
                return MClientList[EP.ToString()];
            }

        }

        public DataTable AllServerStats { get; set; }

        /// <summary>
        /// 获取所有memcached服务器的状态
        /// </summary>
        /// <returns></returns>
        public DataTable GetAllServerStats()
        {
            DataTable ServerStats = new DataTable();
            ServerStats.Columns.Add("Server");
            ServerStats.PrimaryKey = new[] { ServerStats.Columns["Server"] };
            foreach (var server in Servers.ServerList.Values)
            {
                DataRow row = ServerStats.NewRow();
                row["Server"] = string.Format("{0}:{1}", server.Hostname, server.Port);

                foreach (var info in GetServerStats(server))
                {
                    if (!ServerStats.Columns.Contains(info.Key))
                    {
                        ServerStats.Columns.Add(info.Key);
                    }
                    row[info.Key] = info.Value;
                }
                ServerStats.Rows.Add(row);
            }
            AllServerStats = ServerStats;
            return ServerStats;
        }

        /// <summary>
        /// 获取指定服务器状态
        /// </summary>
        /// <param name="server"></param>
        /// <returns></returns>
        public Dictionary<string, string> GetServerStats(TelnetConnection server)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            server.WriteLine("stats");
            string[] statsinfo = Regex.Split(server.Read().Trim(), "STAT");
            foreach (string stat in statsinfo)
            {
                if (stat.Equals(string.Empty)) { continue; }
                string[] info = Regex.Split(stat.Trim(), @"\s+");
                result.Add(info[0], info[1]);
            }
            return result;
        }

        /// <summary>
        /// 返回指定服务器的区块状态
        /// </summary>
        /// <param name="server"></param>
        /// <returns></returns>
        public Dictionary<string, object> StatSlabs(TelnetConnection server)
        {
            return GetinfoByCmd(server, "stats slabs");
        }

        public Dictionary<string, object> StatsItems(TelnetConnection server)
        {
            return GetinfoByCmd(server, "stats items");
        }

        /// <summary>
        /// 格式化输出数据
        /// </summary>
        /// <param name="server"></param>
        /// <param name="cmd"></param>
        /// <returns></returns>
        private Dictionary<string, object> GetinfoByCmd(TelnetConnection server, string cmd)
        {
            Dictionary<string, object> result = new Dictionary<string, object>();
            Dictionary<string, string> infoResult = new Dictionary<string, string>();
            server.WriteLine(cmd);

            string[] statsinfo = cmd.Equals("stats items") ?
                Regex.Split(server.Read().Trim(), "STAT items:")
                : Regex.Split(server.Read().Trim(), "STAT");
            foreach (string stat in statsinfo)
            {
                if (stat.Equals(string.Empty)) { continue; }
                if (stat.Contains(":"))
                {
                    string[] info = Regex.Split(stat.Trim(), ":");
                    if (!result.ContainsKey(info[0]))
                    {
                        infoResult = new Dictionary<string, string>();
                    }

                    string[] slabsinfo = Regex.Split(info[1], @"\s+");
                    infoResult.Add(slabsinfo[0], slabsinfo[1]);
                    result[info[0]] = infoResult;
                }
                else
                {
                    string[] info = Regex.Split(stat.Trim(), @"\s+");
                    if (info.Length > 1)
                    {
                        result.Add(info[0], info[1]);
                    }
                }
            }
            return result;
        }

        public bool StatsReset(TelnetConnection server)
        {
            server.WriteLine("stats reset");
            return server.Read().Trim().CompareTo("RESET") == 0;
        }

        public bool FlushAll(TelnetConnection server)
        {
            server.WriteLine("flush_all");
            return server.Read().Trim().CompareTo("OK") == 0;
        }

        public string Delete(TelnetConnection server,string key)
        {
            server.WriteLine(string.Format("delete {0}",key));
            return server.Read().Trim();
        }

        public Dictionary<string, string> CacheDump(TelnetConnection server, string slab, string count)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();
            server.WriteLine(string.Format("stats cachedump {0} {1}", slab, count));
            string[] CacheInfo = Regex.Split(server.Read().Trim().TrimEnd('E', 'N', 'D'), "ITEM");
            foreach (string cache in CacheInfo)
            {
                if (cache.Equals(string.Empty)) { continue; }
                string[] info = Regex.Split(cache.Trim(), @"\s+\[");
                result.Add(info[0], "[" + info[1]);
            }
            return result;
        }

        /// <summary>
        /// 获取数据
        /// </summary>
        /// <param name="server"></param>
        /// <param name="key"></param>
        /// <returns></returns>
        public Dictionary<string, string> Get(TelnetConnection server, string key)
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            var mc = MClient(new IPEndPoint(IPAddress.Parse(server.Hostname), server.Port));

            object val;
            var r = mc.ExecuteTryGet(key.Trim(), out val);
            if (r.HasValue)
            {
                try
                {
                    result.Add("value", r.Value.ToString());
                }
                catch
                {
                    object obj = r.Value;
                    StringBuilder valToString = new StringBuilder();
                    foreach (PropertyInfo i in obj.GetType().GetProperties())
                    {
                        if (i.PropertyType == typeof(string))
                        {
                            try
                            {
                                valToString.AppendFormat("{0}={1}{2}", i.Name, i.GetValue(obj, new object[0]).ToString(), Environment.NewLine);
                            }
                            catch { }
                        }
                        else if (i.PropertyType.Namespace.Contains("System.Collections"))
                        {
                            valToString.AppendFormat("{0}=List[{2}{1}{2}]{2}", i.Name, ParsingIEnumerable(i.GetValue(obj, null) as IEnumerable), Environment.NewLine);
                        }
                        else
                        {
                            valToString.AppendFormat("{0}={1}{2}", i.Name, i.GetValue(obj, null), Environment.NewLine);
                        }
                    }
                    result.Add("value", valToString.ToString());
                }
                result.Add("type", r.Value.GetType().ToString());
                if (val != null)
                {
                    result["type"] = val.ToString();
                }
            }
            else
            {
                result.Add("value", r.Message);
            }

            return result;
        }

        /// <summary>
        /// 为数据添加单位
        /// </summary>
        /// <param name="datatable"></param>
        /// <returns></returns>
        public DataTable GetUnitTable(DataTable datatable)
        {
            DataTable result = datatable.Copy();

            foreach (DataColumn column in result.Columns)
            {
                if (column.ColumnName.Contains("time"))
                {
                    column.ColumnName += " 秒";
                }
                else if (column.ColumnName.Contains("byte"))
                {
                    column.ColumnName += " 字节";
                }
            }
            return result;
        }

        /// <summary>
        /// 转换迭代器成字符串
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        private string ParsingIEnumerable(IEnumerable obj)
        {
            if (obj.GetType() == typeof(string))
            {
                return obj.ToString();
            }
            StringBuilder valToString = new StringBuilder();
            foreach (var i in obj)
            {
                Type type = i.GetType();
                if (type.IsClass)
                {
                    if (type == typeof(string))
                    {
                        valToString.AppendFormat("{0},", i.ToString());
                    }
                    PropertyInfo[] fieldInfo = i.GetType().GetProperties();
                    if (fieldInfo.Length > 0)
                    {
                        foreach (var child in fieldInfo)
                        {
                            if (!child.CanRead) continue;
                            if (child.PropertyType == typeof(string))
                            {
                                try
                                {
                                    valToString.AppendFormat("{0}={1}{2}", child.Name, child.GetValue(i, null), Environment.NewLine);
                                }
                                catch { }
                            }
                            else if (child.PropertyType.Namespace.Contains("System.Collections"))
                            {
                                valToString.AppendFormat("{0}=List[{2}{1}{2}]{2}", child.Name, ParsingIEnumerable(child.GetValue(i, null) as IEnumerable), Environment.NewLine);
                            }
                            else
                            {
                                valToString.AppendFormat("{0}={1}{2}", child.Name, child.GetValue(i, null).ToString(), Environment.NewLine);
                            }
                        }
                    }
                    else
                    {
                        valToString.AppendFormat("{0},", i.ToString());
                    }
                }
                else
                {
                    valToString.AppendFormat("{0},", i.ToString());
                }
            }
            return valToString.ToString();
        }
    }
}
