﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;


namespace MyMemcached
{
    public partial class MainForm : Form
    {
        private static DAO.Stats stats = new DAO.Stats();
        private static Dictionary<string, object> statSlabs;
        private static Dictionary<string, object> statsItems;

        public static DAO.Stats Stats { get { return stats; } }
        public static Dictionary<string, object> StatSlabs { get { return statSlabs; } }
        public static Dictionary<string, object> StatsItems { get { return statsItems; } }


        public MainForm()
        {
            InitializeComponent();
            this.Focus();
            var servers = System.Configuration.ConfigurationManager.AppSettings["servers"].Split(',');
            AddServers(servers);
        }
        /// <summary>
        /// 打开telnet命令行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void telnetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TelnetCmd telnetCmd = new TelnetCmd();
            telnetCmd.Show();
            telnetCmd.Activate();
        }

        private void btnAddServer_Click(object sender, EventArgs e)
        {
            AddServers(new string[] { string.Format("{0}:{1}", tbHost.Text, tbPort.Text) });
        }

        private void tbPort_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 13)
            {
                btnAddServer_Click(sender, e);
            }
        }

        private void tbHost_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 13)
            {
                btnAddServer_Click(sender, e);
            }
        }

        private void ServerList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ServerList.SelectedItem == null)
            {
                return;
            }
            Servers.CurrentServer = Servers.ServerList[ServerList.SelectedItem.ToString()];
            DataBind();
        }
        private void cmbSlabs_SelectedIndexChanged(object sender, EventArgs e)
        {
            SlabsInfoBind(statSlabs, statsItems);
        }

        private void DataBind()
        {
            if (ServerList.SelectedItem == null)
            {
                return;
            }
            statSlabs = stats.StatSlabs(Servers.CurrentServer);
            statsItems = stats.StatsItems(Servers.CurrentServer);
            cmbSlabs.ResetText();
            cmbSlabs.Items.Clear();
            foreach (string slab in statSlabs.Keys)
            {
                if (slab.Equals("active_slabs") || slab.Equals("total_malloced"))
                {
                    continue;
                }
                cmbSlabs.Items.Add(slab);
            }

            ProgressDataBind();

            if (cmbSlabs.Items.Count > 0)
            {
                cmbSlabs.SelectedIndex = 0;
            }
            SlabsInfoBind(statSlabs, statsItems);
        }

        //绑定进度条数据
        private void ProgressDataBind()
        {
            if (ServerList.SelectedItem == null)
            {
                return;
            }
            DataRow serverStats = stats.AllServerStats.Rows.Find(ServerList.SelectedItem.ToString());

            pbbytes.Maximum = Convert.ToInt32(serverStats["limit_maxbytes"]);
            pbbytes.Value = Convert.ToInt32(serverStats["bytes"]);

            pbget.Maximum = Convert.ToInt32(serverStats["cmd_get"]);
            pbget.Value = Convert.ToInt32(serverStats["get_hits"]);

            pbDelete.Maximum = Convert.ToInt32(serverStats["delete_hits"]) +
                Convert.ToInt32(serverStats["delete_misses"]);
            pbDelete.Value = Convert.ToInt32(serverStats["delete_hits"]);

            pbIncr.Maximum = Convert.ToInt32(serverStats["incr_hits"]) +
                Convert.ToInt32(serverStats["incr_misses"]);
            pbIncr.Value = Convert.ToInt32(serverStats["incr_hits"]);

            pbDecr.Maximum = Convert.ToInt32(serverStats["decr_hits"]) +
                Convert.ToInt32(serverStats["decr_misses"]);
            pbDecr.Value = Convert.ToInt32(serverStats["decr_hits"]);

            pbCAS.Maximum = Convert.ToInt32(serverStats["cas_hits"]) +
                Convert.ToInt32(serverStats["cas_misses"]);
            pbCAS.Value = Convert.ToInt32(serverStats["cas_hits"]);
        }

        //textbox数据绑定
        private void SlabsInfoBind(Dictionary<string, object> statSlabs, Dictionary<string, object> statsItems)
        {
            lbActiveSlabs.Text = "slab数量: " + statSlabs["active_slabs"].ToString();
            lbTotalMalloced.Text = "总内存数量: " + statSlabs["total_malloced"].ToString() + " 字节";

            slabView.ResetText();
            itemsview.ResetText();
            if (cmbSlabs.Items.Count <= 0)
            { return; }

            foreach (var slabinfo in (Dictionary<string, string>)statSlabs[cmbSlabs.SelectedItem.ToString()])
            {
                slabView.Text += string.Format("{0}:\t{1}\r\n", slabinfo.Key, slabinfo.Value);
            }

            if (!statsItems.ContainsKey(cmbSlabs.SelectedItem.ToString()))
            {
                return;
            }
            foreach (var itemsinfo in (Dictionary<string, string>)statsItems[cmbSlabs.SelectedItem.ToString()])
            {
                itemsview.AppendText(string.Format("{0}:\t{1}\r\n", itemsinfo.Key, itemsinfo.Value));
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            dataGridView1.DataSource = stats.GetUnitTable(stats.GetAllServerStats());
            DataBind();
        }

        private void statsResetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string message = string.Format("重置{0}:{1}的状态?", Servers.CurrentServer.Hostname, Servers.CurrentServer.Port);
            if (MessageBox.Show(message, "Reset", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                if (stats.StatsReset(Servers.CurrentServer))
                {
                    refreshToolStripMenuItem_Click(sender, e);
                }
            }
        }

        private void flushallToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string message = string.Format("清空{0}:{1}所有的缓存?", Servers.CurrentServer.Hostname, Servers.CurrentServer.Port);
            if (MessageBox.Show(message, "Flush_ALL", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                if (stats.FlushAll(Servers.CurrentServer))
                {
                    refreshToolStripMenuItem_Click(sender, e);
                }
            }
        }

        private void DataTraversalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (statSlabs == null)
            {
                MessageBox.Show("请选择服务器");
                return;
            }
            DataTraversal dataTraversal = new DataTraversal();
            dataTraversal.Show();
            dataTraversal.Activate();
        }

        private void getToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (statSlabs == null)
            {
                MessageBox.Show("请选择服务器");
                return;
            }
            GetForm getForm = new GetForm();
            getForm.Show();
            getForm.Activate();
        }

        private void AddServers(string[] servers)
        {
            try
            {
                foreach (string server in servers)
                {
                    try
                    {
                        string[] val = server.Split(':');
                        TelnetConnection telnet = new TelnetConnection(val[0].Trim(), Convert.ToInt32(val[1].Trim()));
                        Servers.Add(telnet);
                        ServerList.Items.Add(string.Format("{0}:{1}", telnet.Hostname, telnet.Port));
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
                dataGridView1.DataSource = stats.GetUnitTable(stats.GetAllServerStats());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        //程序退出
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            foreach (var client in MyMemcached.DAO.Stats.MClientList)
            {
                client.Value.Dispose();
            }

            foreach (var server in MyMemcached.Servers.ServerList)
            {
                server.Value.DisConnect();
            }
        }
    }
}
