﻿using System;
using System.Configuration;
using Enyim.Caching.Configuration;
using Enyim.Caching.Memcached;
using System.Collections.Generic;
using System.Threading;
using System.Net;
using System.Diagnostics;
using Enyim.Caching;
using Enyim.Caching.Memcached.Results;
using Enyim.Caching.Memcached.Results.Factories;
using Enyim.Caching.Memcached.Results.Extensions;
using System.Text;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text.RegularExpressions;

namespace MyMemcached
{
    public class mcacheClientExt : MemcachedClient
    {
        public mcacheClientExt():base()
        {
        }
        public mcacheClientExt(MemcachedClientConfiguration config)
            : base(config)
        {
        }

        protected override IGetOperationResult PerformTryGet(string key, out ulong cas, out object value)
        {
            var hashedKey = base.KeyTransformer.Transform(key);
            var node = base.Pool.Locate(hashedKey);
            var result = GetOperationResultFactory.Create();

            cas = 0;
            value = null;

            if (node != null)
            {
                var command = base.Pool.OperationFactory.Get(hashedKey);
                var commandResult = node.Execute(command);

                if (commandResult.Success)
                {
                    try
                    {
                        result.Value = base.Transcoder.Deserialize(command.Result);
                        value = null;
                    }
                    catch
                    {
                        result.Success = false;
                        result.Value = "找不到该缓存所在的程序集.";

                        //经调试发现,就算没有相应的DLL ,command.Result.Data.Array可以获取该值的类型
                        value = "估计类型:" + Regex.Split(Encoding.UTF8.GetString(command.Result.Data.Array).Trim(), @"\0+")[7];
                    }
                   
                    result.Cas = cas = command.CasValue;

                    if (base.PerformanceMonitor != null)
                        base.PerformanceMonitor.Get(1, true);

                    //返回时 用IP地址替换result.Message(默认null)
                    result.Pass(node.EndPoint.ToString());
                    return result;
                }
                else
                {
                    commandResult.Combine(result);
                    return result;
                }
            }

            result.Value = value;
            result.Cas = cas;

            if (base.PerformanceMonitor != null)
                base.PerformanceMonitor.Get(1, false);

            result.Fail("Unable to locate node");
            return result;
        }
    }
}
